﻿using BoyHowdy.Erb.Lib.RuleEngine;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using BoyHowdy.Erb.Lib.RuleSets;
using BoyHowdy.Erb.Tests.Data;

namespace BoyHowdy.Erb.Tests
{


    /// <summary>
    ///This is a test class for RuleParserTest and is intended
    ///to contain all RuleParserTest Unit Tests
    ///</summary>
    [TestClass()]
    public class SurveyRuleParserTest
    {
        List<string> rules;
        public TestContext TestContext { get; set; }

        public SurveyRuleParserTest()
        {
            rules = new List<string> { "If answer to question 1 is Very Unsatisfied", 
                                                    "Then send survey via email to clientcarespecialist@keithelder.net", 
                                                    "If answer to question 3 is Yes", 
                                                    "Then subscribe them to newsletter", 
                                                    "If answer to question 3 is Yes", 
                                                    "And client is a repeat client",
                                                    "Then send them a discount coupon in the mail",
                                                    "If answer to question 4 is Yes",
                                                    "Then schedule a follow up call in 6 months",
                                                    };
        }

        [TestMethod]
        public void Survey_Rules_Should_Parse_Into_Code_And_Execute_For_Satified_Survey()
        {

            foreach (var rule in rules)
            {
                TestContext.WriteLine(rule);
            }

            TestContext.WriteLine("");

            var code = RuleParser.ToRubyScript<SurveyRuleSet>(rules);
            TestContext.WriteLine("{0}", code);
            Assert.IsTrue(code.Length > 0);

            var ctx = SurveyGenerator.GetSatisfiedClientSurveyData();
            ctx.RepeatClient = true;
            var ruleSet = new SurveyRuleSet(ctx);
            Engine.Execute<SurveyRuleSet>(code, ruleSet);
            Assert.IsTrue(ruleSet.Context.FollowUpInSixMonths == true);
            Assert.IsTrue(ruleSet.Context.SendDiscountCouponInMail == true);
            Assert.IsTrue(ruleSet.Context.SubScribedToNewsLetter == true);

        }


        [TestMethod]
        public void Survey_Rules_Should_Parse_Into_Code_And_Execute_For_UnSatified_Survey()
        {

            foreach (var rule in rules)
            {
                TestContext.WriteLine(rule);
            }

            TestContext.WriteLine("");

            var code = RuleParser.ToRubyScript<SurveyRuleSet>(rules);
            TestContext.WriteLine("{0}", code);
            Assert.IsTrue(code.Length > 0);


            var ctx = SurveyGenerator.GetUnSatisfiedClientSurveyData();
            ctx.RepeatClient = true;
            var ruleSet = new SurveyRuleSet(ctx);
            Engine.Execute<SurveyRuleSet>(code, ruleSet);
            Assert.IsTrue(ruleSet.Context.FollowUpInSixMonths == false);
            Assert.IsTrue(ruleSet.Context.SendDiscountCouponInMail == false);
            Assert.IsTrue(ruleSet.Context.SubScribedToNewsLetter == false);
            Assert.IsTrue(ruleSet.Context.EmailedClientSpecialist == true);
        }
    }
}
