﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Reflection;
using BoyHowdy.Erb.Lib.Common;
using System.Diagnostics;

namespace BoyHowdy.Erb.Lib.RuleEngine
{
    public class Engine
    {
        public static IEnumerable<Type> FindAllRuleSetTypes()
        {
            var type = typeof(IRuleSet);
            var types = AppDomain.CurrentDomain.GetAssemblies().ToList()
                .SelectMany(s => s.GetTypes())
                .Where(p => type.IsAssignableFrom(p));
            return types;
        }

        public static IEnumerable<string> GetAllRulesForRuleSet(Type ruleSet)
        {
            List<string> rules = new List<string>();
            if (ruleSet != null)
            {
                foreach (var method in ruleSet.GetMethods())
                {
                    var attributes = method.GetCustomAttributes(typeof(RuleAttribute), true);
                    foreach (var attr in attributes)
                    {
                        var attrType = attr.GetType();
                        string key = string.Format("{0} {1}", attrType.Name.Replace("Attribute", string.Empty), ((RuleAttribute)attr).Regex);
                        rules.Add(key);
                    }
                }
            }
            return rules;
        }

        public static void Execute<IRuleSet>(string rubyScript, IRuleSet ruleSet)
        {
            Log.WriteLine("Initialiing ruby script engine");
            var watch = Stopwatch.StartNew();
            var engine = IronRuby.Ruby.CreateEngine();
            var scope = engine.CreateScope();
            scope.SetVariable("ruleSet", ruleSet);
            watch.Stop();
            Log.WriteLine("Ruby script engine created in " + watch.ElapsedMilliseconds + " ms");
            Log.WriteLine("Executing ruby script");
            var watch2 = Stopwatch.StartNew();
            engine.Execute(rubyScript, scope);
            watch.Stop();
            Log.WriteLine("Script executed in " + watch2.ElapsedMilliseconds + " ms");
        }
    }
}
