﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Reflection;

namespace BoyHowdy.Erb.Lib.RuleEngine
{
    public class RuleParser
    {
        /// <summary>
        /// Takes a list of rules and parses them into Ruby script. Note that while this works 
        /// it could def use some refactoring. 
        /// </summary>
        /// <param name="rules">The rules.</param>
        /// <returns></returns>
        public static string ToRubyScript<T>(List<string> rules)
        {
            Dictionary<Regex, KeyValuePair<Type, MethodInfo>> keyMap = BuildMapOfTextToFunctions<T>();

            var sb = new StringBuilder();
            bool ifOpened = false;
            foreach (var rule in rules)
            {
                if (string.IsNullOrEmpty(rule))
                {
                    continue;
                }

                var match = keyMap.Where(x => x.Key.IsMatch(rule)).FirstOrDefault();

                if (match.Key == null)
                {
                    throw new Exception("No match for rule found");
                }

                var matches = match.Key.Match(rule);
                if (matches == null)
                {
                    throw new Exception("Invalid Rule");
                }

                var groups = matches.Groups;
                if (groups.Count <= 1)
                {
                    if (match.Value.Value != null)
                    {
                        if (ifOpened)
                        {
                            if (match.Value.Key.Name == "AndAttribute")
                            {
                                sb.Append(String.Format(" and ruleSet.{0}()", match.Value.Value.Name));
                                continue;
                            }

                            if (match.Value.Key.Name == "OrAttribute")
                            {
                                sb.Append(String.Format(" or ruleSet.{0}()", match.Value.Value.Name));
                                continue;
                            }
                            sb.AppendLine();
                            sb.Append("\t");

                        }
                        sb.Append(string.Format("ruleSet.{0}()", match.Value.Value.Name));
                    }
                    continue;
                }


                // parse the string for parameters
                if (match.Value.Value.GetParameters().Length == 1)
                {
                    if (match.Value.Key.Name == "IfAttribute" || match.Value.Key.Name == "SiAttribute")
                    //if (match.Value.Key.Name == "IfAttribute")
                    {
                        if (ifOpened)
                        {
                            sb.AppendLine();
                            sb.AppendLine("end");
                        }
                        //if (andOpened) throw new Exception("Error parsing rules");
                        ifOpened = true;
                        sb.Append(String.Format("if ruleSet.{0}(", match.Value.Value.Name));
                    }

                    if (match.Value.Key.Name == "ThenAttribute")
                    {
                        sb.AppendLine();
                        sb.Append("\t");
                        sb.Append(String.Format("ruleSet.{0}(", match.Value.Value.Name));
                    }

                    if (match.Value.Key.Name == "AndAttribute")
                    {
                        sb.Append(String.Format(" and ruleSet.{0}(", match.Value.Value.Name));
                    }

                    if (match.Value.Key.Name == "OrAttribute")
                    {
                        sb.Append(String.Format(" or ruleSet.{0}(", match.Value.Value.Name));
                    }

                    var count = groups.Count;
                    string param = String.Empty;
                    string tail = ", ";
                    for (int i = 1; i < count; i++)
                    {
                        if (i == count - 1)
                        {
                            tail = String.Empty;
                        }

                        param += String.Format(@"""{0}""{1}", groups[i], tail);
                    }
                    sb.Append(string.Format("{1})", match.Value.Value.Name, param));
                    continue;
                }


                if (match.Value.Value != null)
                {
                    sb.Append(string.Format(".{0}()", match.Value.Value.Name));
                }
            }
            if (ifOpened)
            {
                sb.AppendLine();
                sb.AppendLine("end");
            }
            return sb.ToString();
        }

        private static Dictionary<Regex, KeyValuePair<Type, MethodInfo>> BuildMapOfTextToFunctions<T>()
        {
            Dictionary<Regex, KeyValuePair<Type, MethodInfo>> keyMap = new Dictionary<Regex, KeyValuePair<Type, MethodInfo>>();

            var type = typeof(T);
            foreach (var method in type.GetMethods())
            {
                var attributes = method.GetCustomAttributes(typeof(RuleAttribute), true);
                foreach (var attr in attributes)
                {
                    var attrType = attr.GetType();
                    string key = string.Format("{0} {1}", attrType.Name.Replace("Attribute", string.Empty), ((RuleAttribute)attr).Regex);

                    key = key.Replace(" ", "\\s");
                    keyMap.Add(new Regex(key), new KeyValuePair<Type, MethodInfo>(attrType, method));
                }
            }
            return keyMap;
        }
    }
}
