﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using BoyHowdy.Erb.Lib.RuleSets;

namespace BoyHowdy.Erb.Lib.RuleEngine
{
    public class RuleSetBase : IRuleSet
    {
        public dynamic Context { get; set; }

        /// <summary>
        /// Initializes a new instance of the RuleSetBase class.
        /// </summary>
        public RuleSetBase(dynamic context)
        {
            Context = context;
        }

        /// <summary>
        /// Initializes a new instance of the RuleSetBase class.
        /// </summary>
        public RuleSetBase()
        {
            
        }

        /// <summary>
        /// Evaluates if a property of the given context is equal to the value specified.
        /// </summary>
        /// <remarks>This does not support null or false. Needs to be added.</remarks>
        /// <param name="args">The args.</param>
        /// <returns></returns>
        /// <example>Example english rule: If Age equals 30</example>
        [If(@"(\w+) equals (.+)")]
        public bool IfPropertyEquals(params string[] args) 
        {
            var property = args[0];
            var value = args[1];
            var prop = Context.GetType().GetProperty(property);
            if (prop == null) return false;
            if (prop.PropertyType.Name == "Int32")
            {
                int outValue;
                if (Int32.TryParse(value, out outValue))
                {
                    var reflectedValue = prop.GetValue(Context, null);

                    return reflectedValue == outValue;
                }
                return false;
            }

            if (prop.PropertyType.Name == "Int64")
            {
                long outValue;
                if (Int64.TryParse(value, out outValue))
                {
                    var reflectedValue = prop.GetValue(Context, null);

                    return reflectedValue == outValue;
                }
                return false;
            }

            if (prop.PropertyType.Name == "String")
            {
                    var reflectedValue = prop.GetValue(Context, null);
                    return reflectedValue == value;
            }
            return false;
        }
        
    }
}

