﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Runtime.Serialization;
using BoyHowdy.Erb.Lib.RuleEngine;
using BoyHowdy.Erb.Lib.Domain.Survey;
using System.Web.Mail;
using BoyHowdy.Erb.Lib.Common;


namespace BoyHowdy.Erb.Lib.RuleSets
{
    /// <summary>
    /// This rule set is just a fun example of showing how rulesets can match the domain or business language spoken.
    /// </summary>
    public class SouthernRuleSet : RuleSetBase
    {
        public Survey Context { get; set; }

        public SouthernRuleSet(Survey context)
        {
            Context      = context;
            base.Context = (dynamic)context;
        }

        [IfAttribute(@"they done gone and answered question (\d+) as (.+)")]
        public bool IfAnswerToQuestion(params string[] args)
        {
            return DoesQuestionHaveAnswer(args[0], args[1]);
        }

        [Then(@"subscribe 'em to that there newsletter")]
        public void ThenSubscribeThemToNewsLetter()
        {
            // call into domain
            Log.WriteLine("Getting our newsletter on boy!");
            Context.SubScribedToNewsLetter = true;
        }

        [And(@"client done been here before")]
        public bool AndClientIsRepeatClient()
        {
            Log.WriteLine("Checking if client has done gone and been here before");
            return Context.RepeatClient;
        }

        [Then(@"send 'em one of them there fancy discount coupons")]
        public void ThenSendThemADiscountCoupon()
        {
            Log.WriteLine("Sending 'em one of them there discount coupons in the mail");
            Context.SendDiscountCouponInMail = true;
        }

        [Then(@"use that calling device and call 'em back in (\d+) months")]
        public void ThenScheduleFollowUpCall(params string[] args)
        {
            Context.FollowUpInSixMonths = true;
            Log.WriteLine(String.Format("Need to call 'em back in {0} months", args[0]));
        }

        [Then(@"send one of those electronic mails to (.+)")]
        public void ThenSendAnEmailTo(params string[] args)
        {
            Context.EmailedClientSpecialist = true;
            Log.WriteLine("Sending one of them there electronic mails to " + args[0]);
        }


        [And(@"they done gone and answered question (\d+) as (.+)")]
        public bool AndAnswerToQuestion(params string[] args)
        {
            return DoesQuestionHaveAnswer(args[0], args[1]);
        }

        [Or(@"they done gone and answered question (\d+) as (.+)")]
        public bool OrAnswerToQuestion(params string[] args)
        {
            return DoesQuestionHaveAnswer(args[0], args[1]);
        }

        private bool DoesQuestionHaveAnswer(string questionNumber, string answer)
        {
            var questionNum = Convert.ToInt32(questionNumber);
            var q = (from qa in Context.Questions
                     where qa.Number == questionNum
                     && qa.Answer == answer
                     select qa.Answer).FirstOrDefault();
            if (q != null)
            {
                Log.WriteLine("Question " + questionNumber + " did gone and matched answer: " + answer);
                return true;
            }
            Log.WriteLine("Well slap my Momma, question " + questionNumber + " didn't dad burn match answer: " + answer);
            return false;
        }
        
    }
}
