﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Runtime.Serialization;
using BoyHowdy.Erb.Lib.RuleEngine;
using BoyHowdy.Erb.Lib.Domain.Survey;
using System.Web.Mail;
using BoyHowdy.Erb.Lib.Common;


namespace BoyHowdy.Erb.Lib.RuleSets
{
    public class SurveyRuleSet : RuleSetBase
    {
        public Survey Context { get; set; }

        public SurveyRuleSet(Survey context)
        {
            Context      = context;
            base.Context = (dynamic)context;
        }

        [IfAttribute(@"answer to question (\d+) is (.+)")]
        //[IfAttribute(@"answer to question (\d+) contains (.+)")]
        //[IfAttribute(@"answer to question (\d+) has the value of (.+)")]
        //[SiAttribute(@"la respuesta para preguntar (\d+) es (.+)")] // Es Satisfecho Muy
        public bool IfAnswerToQuestion(params string[] args)
        {
            return DoesQuestionHaveAnswer(args[0], args[1]);
        }

        [Then(@"subscribe them to newsletter")]
        public void ThenSubscribeThemToNewsLetter()
        {
            Log.WriteLine("Subscribing to news letter");
            Context.SubScribedToNewsLetter = true;
        }

        [And(@"client is a repeat client")]
        public bool AndClientIsRepeatClient()
        {
            Log.WriteLine("Checking if client is a repeat client");
            return Context.RepeatClient;
        }

        [Then(@"send them a discount coupon in the mail")]
        public void ThenSendThemADiscountCoupon()
        {
            Log.WriteLine("Sending them a discount coupon in the mail");
            Context.SendDiscountCouponInMail = true;
        }

        [Then(@"schedule a follow up call in (\d+) months")]
        public void ThenScheduleFollowUpCall(params string[] args)
        {
            Context.FollowUpInSixMonths = true;
            Log.WriteLine(String.Format("Scheduling follow up call in {0} months", args[0]));
        }

        [Then(@"send survey via email to (.+)")]
        public void ThenSendAnEmailTo(params string[] args)
        {
            Context.EmailedClientSpecialist = true;
            Log.WriteLine("Sending an email to " + args[0]);
        }


        [And(@"answer to question (\d+) is (.+)")]
        public bool AndAnswerToQuestion(params string[] args)
        {
            return DoesQuestionHaveAnswer(args[0], args[1]);
        }

        [Or(@"answer to question (\d+) is (.+)")]
        public bool OrAnswerToQuestion(params string[] args)
        {
            return DoesQuestionHaveAnswer(args[0], args[1]);
        }

        private bool DoesQuestionHaveAnswer(string questionNumber, string answer)
        {
            var questionNum = Convert.ToInt32(questionNumber);
            var q = (from qa in Context.Questions
                     where qa.Number == questionNum
                     && qa.Answer == answer
                     select qa.Answer).FirstOrDefault();
            if (q != null)
            {
                Log.WriteLine("Question " + questionNumber + " did match answer: " + answer);
                return true;
            }
            Log.WriteLine("Question " + questionNumber + " did NOT match answer: " + answer);
            return false;
        }
        
    }
}
